@@ -118,22 +118,23 @@ func TestSetupData(t *testing.T) {
 
 			piState := getTestPreInitState(t)
 			runner, err := js.New(
-				piState, &loader.SourceData{URL: &url.URL{Path: "/script.js"}, Data: testCase.script}, nil,
+				piState,
+				&loader.SourceData{
+					URL:  &url.URL{Path: "/script.js"},
+					Data: testCase.script,
+				},
+				nil,
 			)
 			require.NoError(t, err)
-			require.NoError(t, runner.SetOptions(lib.Options{
+
+			testState := getTestRunState(t, lib.Options{
 				Paused:          null.BoolFrom(true),
 				VUs:             null.IntFrom(2),
 				Iterations:      null.IntFrom(3),
 				NoSetup:         null.BoolFrom(true),
 				SetupTimeout:    types.NullDurationFrom(5 * time.Second),
 				TeardownTimeout: types.NullDurationFrom(5 * time.Second),
-			}))
-			testState := &lib.TestRunState{
-				TestPreInitState: piState,
-				Options:          runner.GetOptions(),
-				Runner:           runner,
-			}
+			}, runner)
 
 			execScheduler, err := local.NewExecutionScheduler(testState)
 			require.NoError(t, err)
