@@ -1,22 +1,3 @@
-/*
- *
- * k6 - a next-generation load testing tool
- * Copyright (C) 2019 Load Impact
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- */
 package cmd
 
 import (
@@ -370,6 +351,20 @@ func getConfigConsolidationTestCases() []configConsolidationTestCase {
 				)
 			},
 		},
+
+		// Test-wide Tags
+		{
+			opts{
+				fs:  defaultConfig(`{"tags": { "codeTagKey": "codeTagValue"}}`),
+				cli: []string{"--tag", "clitagkey=clitagvalue"},
+			},
+			exp{},
+			func(t *testing.T, c Config) {
+				exp := map[string]string{"clitagkey": "clitagvalue"}
+				assert.Equal(t, exp, c.RunTags)
+			},
+		},
+
 		// Test summary trend stats
 		{opts{}, exp{}, func(t *testing.T, c Config) {
 			assert.Equal(t, lib.DefaultSummaryTrendStats, c.Options.SummaryTrendStats)
