@@ -52,6 +52,7 @@ func getTestRunState(
 		TestPreInitState: piState,
 		Options:          options,
 		Runner:           runner,
+		RunTags:          piState.Registry.RootTagSet().WithTagsFromMap(options.RunTags),
 	}
 }
 
@@ -335,29 +336,19 @@ func TestExecutionSchedulerSystemTags(t *testing.T) {
 		require.NoError(t, execScheduler.Run(ctx, ctx, samples))
 	}()
 
-	expCommonTrailTags := metrics.IntoSampleTags(&map[string]string{
-		"group":             "",
-		"method":            "GET",
-		"name":              sr("HTTPBIN_IP_URL/"),
-		"url":               sr("HTTPBIN_IP_URL/"),
-		"proto":             "HTTP/1.1",
-		"status":            "200",
-		"expected_response": "true",
-	})
-	expTrailPVUTagsRaw := expCommonTrailTags.CloneTags()
-	expTrailPVUTagsRaw["scenario"] = "per_vu_test"
-	expTrailPVUTags := metrics.IntoSampleTags(&expTrailPVUTagsRaw)
-	expTrailSITagsRaw := expCommonTrailTags.CloneTags()
-	expTrailSITagsRaw["scenario"] = "shared_test"
-	expTrailSITags := metrics.IntoSampleTags(&expTrailSITagsRaw)
-	expNetTrailPVUTags := metrics.IntoSampleTags(&map[string]string{
-		"group":    "",
-		"scenario": "per_vu_test",
-	})
-	expNetTrailSITags := metrics.IntoSampleTags(&map[string]string{
-		"group":    "",
-		"scenario": "shared_test",
-	})
+	expCommonTrailTags := testRunState.RunTags.
+		With("group", "").
+		With("method", "GET").
+		With("name", sr("HTTPBIN_IP_URL/")).
+		With("url", sr("HTTPBIN_IP_URL/")).
+		With("proto", "HTTP/1.1").
+		With("status", "200").
+		With("expected_response", "true")
+
+	expTrailPVUTags := expCommonTrailTags.With("scenario", "per_vu_test")
+	expTrailSITags := expCommonTrailTags.With("scenario", "shared_test")
+	expNetTrailPVUTags := testRunState.RunTags.With("group", "").With("scenario", "per_vu_test")
+	expNetTrailSITags := testRunState.RunTags.With("group", "").With("scenario", "shared_test")
 
 	var gotCorrectTags int
 	for {
@@ -365,16 +356,17 @@ func TestExecutionSchedulerSystemTags(t *testing.T) {
 		case sample := <-samples:
 			switch s := sample.(type) {
 			case *httpext.Trail:
-				if s.Tags.IsEqual(expTrailPVUTags) || s.Tags.IsEqual(expTrailSITags) {
+				if s.Tags == expTrailPVUTags || s.Tags == expTrailSITags {
 					gotCorrectTags++
 				}
 			case *netext.NetTrail:
-				if s.Tags.IsEqual(expNetTrailPVUTags) || s.Tags.IsEqual(expNetTrailSITags) {
+				if s.Tags == expNetTrailPVUTags || s.Tags == expNetTrailSITags {
 					gotCorrectTags++
 				}
 			}
+
 		case <-done:
-			require.Equal(t, 4, gotCorrectTags, "received wrong amount of samples with expected tags")
+			assert.Equal(t, 4, gotCorrectTags, "received wrong amount of samples with expected tags")
 			return
 		}
 	}
@@ -479,13 +471,13 @@ func TestExecutionSchedulerRunCustomTags(t *testing.T) {
 				select {
 				case sample := <-samples:
 					if trail, ok := sample.(*httpext.Trail); ok && !gotTrailTag {
-						tags := trail.Tags.CloneTags()
+						tags := trail.Tags.Map()
 						if v, ok := tags["customTag"]; ok && v == "value" {
 							gotTrailTag = true
 						}
 					}
 					if netTrail, ok := sample.(*netext.NetTrail); ok && !gotNetTrailTag {
-						tags := netTrail.Tags.CloneTags()
+						tags := netTrail.Tags.Map()
 						if v, ok := tags["customTag"]; ok && v == "value" {
 							gotNetTrailTag = true
 						}
@@ -661,7 +653,7 @@ func TestExecutionSchedulerRunCustomConfigNoCrossover(t *testing.T) {
 				assert.FailNow(t, "received error sample from test")
 			}
 			if s.Metric.Name == "checks" || s.Metric.Name == "group_duration" {
-				tags := s.Tags.CloneTags()
+				tags := s.Tags.Map()
 				for _, expTags := range expectedPlainSampleTags {
 					if reflect.DeepEqual(expTags, tags) {
 						gotSampleTags++
@@ -669,14 +661,14 @@ func TestExecutionSchedulerRunCustomConfigNoCrossover(t *testing.T) {
 				}
 			}
 		case *httpext.Trail:
-			tags := s.Tags.CloneTags()
+			tags := s.Tags.Map()
 			for _, expTags := range expectedTrailTags {
 				if reflect.DeepEqual(expTags, tags) {
 					gotSampleTags++
 				}
 			}
 		case *netext.NetTrail:
-			tags := s.Tags.CloneTags()
+			tags := s.Tags.Map()
 			for _, expTags := range expectedNetTrailTags {
 				if reflect.DeepEqual(expTags, tags) {
 					gotSampleTags++
@@ -684,7 +676,7 @@ func TestExecutionSchedulerRunCustomConfigNoCrossover(t *testing.T) {
 			}
 		case metrics.ConnectedSamples:
 			for _, sm := range s.Samples {
-				tags := sm.Tags.CloneTags()
+				tags := sm.Tags.Map()
 				if reflect.DeepEqual(expectedConnSampleTags, tags) {
 					gotSampleTags++
 				}
@@ -880,7 +872,7 @@ func TestExecutionSchedulerRuntimeErrors(t *testing.T) {
 	assert.True(t, isFinal)
 
 	startTime := time.Now()
-	assert.NoError(t, execScheduler.Run(ctx, ctx, samples))
+	require.NoError(t, execScheduler.Run(ctx, ctx, samples))
 	runTime := time.Since(startTime)
 	assert.True(t, runTime > 1*time.Second, "test did not take 1s")
 	assert.True(t, runTime < 10*time.Second, "took more than 10 seconds")
@@ -927,7 +919,12 @@ func TestExecutionSchedulerEndErrors(t *testing.T) {
 
 func TestExecutionSchedulerEndIterations(t *testing.T) {
 	t.Parallel()
-	metric := &metrics.Metric{Name: "test_metric"}
+	registry := metrics.NewRegistry()
+	metric := registry.MustNewMetric("test_metric", metrics.Counter)
+	ts := metrics.TimeSeries{
+		Metric: metric,
+		Tags:   registry.RootTagSet(),
+	}
 
 	options, err := executor.DeriveScenariosFromShortcuts(lib.Options{
 		VUs:        null.IntFrom(1),
@@ -944,7 +941,10 @@ func TestExecutionSchedulerEndIterations(t *testing.T) {
 			default:
 				atomic.AddInt64(&i, 1)
 			}
-			out <- metrics.Sample{Metric: metric, Value: 1.0}
+			out <- metrics.Sample{
+				TimeSeries: ts,
+				Value:      1.0,
+			}
 			return nil
 		},
 		Options: options,
@@ -968,7 +968,7 @@ func TestExecutionSchedulerEndIterations(t *testing.T) {
 	for i := 0; i < 100; i++ {
 		mySample, ok := <-samples
 		require.True(t, ok)
-		assert.Equal(t, metrics.Sample{Metric: metric, Value: 1.0}, mySample)
+		assert.Equal(t, metrics.Sample{TimeSeries: ts, Value: 1.0}, mySample)
 	}
 }
 
@@ -1206,7 +1206,7 @@ func TestRealTimeAndSetupTeardownMetrics(t *testing.T) {
 							assert.Equal(t, expS.Value, s.Value)
 						}
 						assert.Equal(t, expS.Metric.Name, s.Metric.Name)
-						assert.Equal(t, expS.Tags.CloneTags(), s.Tags.CloneTags())
+						assert.Equal(t, expS.Tags.Map(), s.Tags.Map())
 						assert.InDelta(t, 0, now.Sub(s.Time), float64(50*time.Millisecond))
 					}
 				}
@@ -1218,31 +1218,35 @@ func TestRealTimeAndSetupTeardownMetrics(t *testing.T) {
 		}
 	}
 
-	getTags := func(args ...string) *metrics.SampleTags {
-		tags := map[string]string{}
+	getTags := func(r *metrics.Registry, args ...string) *metrics.TagSet {
+		tags := r.RootTagSet()
 		for i := 0; i < len(args)-1; i += 2 {
-			tags[args[i]] = args[i+1]
+			tags = tags.With(args[i], args[i+1])
 		}
-		return metrics.IntoSampleTags(&tags)
+		return tags
 	}
 	testCounter, err := piState.Registry.NewMetric("test_counter", metrics.Counter)
 	require.NoError(t, err)
 	getSample := func(expValue float64, expMetric *metrics.Metric, expTags ...string) metrics.SampleContainer {
 		return metrics.Sample{
-			Metric: expMetric,
-			Time:   time.Now(),
-			Tags:   getTags(expTags...),
-			Value:  expValue,
+			TimeSeries: metrics.TimeSeries{
+				Metric: expMetric,
+				Tags:   getTags(piState.Registry, expTags...),
+			},
+			Time:  time.Now(),
+			Value: expValue,
 		}
 	}
 	getDummyTrail := func(group string, emitIterations bool, addExpTags ...string) metrics.SampleContainer {
 		expTags := []string{"group", group}
 		expTags = append(expTags, addExpTags...)
-		return netext.NewDialer(
+		dialer := netext.NewDialer(
 			net.Dialer{},
 			netext.NewResolver(net.LookupIP, 0, types.DNSfirst, types.DNSpreferIPv4),
-		).GetTrail(time.Now(), time.Now(),
-			true, emitIterations, getTags(expTags...), piState.BuiltinMetrics)
+		)
+
+		ctm := metrics.TagsAndMeta{Tags: getTags(piState.Registry, expTags...)}
+		return dialer.GetTrail(time.Now(), time.Now(), true, emitIterations, ctm, piState.BuiltinMetrics)
 	}
 
 	// Initially give a long time (5s) for the execScheduler to start
