@@ -105,12 +105,16 @@ func (mi *K6) Group(name string, fn goja.Callable) (goja.Value, error) {
 
 	shouldUpdateTag := state.Options.SystemTags.Has(metrics.TagGroup)
 	if shouldUpdateTag {
-		state.Tags.Set("group", g.Path)
+		state.Tags.Modify(func(tagsAndMeta *metrics.TagsAndMeta) {
+			tagsAndMeta.SetSystemTagOrMeta(metrics.TagGroup, g.Path)
+		})
 	}
 	defer func() {
 		state.Group = old
 		if shouldUpdateTag {
-			state.Tags.Set("group", old.Path)
+			state.Tags.Modify(func(tagsAndMeta *metrics.TagsAndMeta) {
+				tagsAndMeta.SetSystemTagOrMeta(metrics.TagGroup, old.Path)
+			})
 		}
 	}()
 
@@ -118,14 +122,16 @@ func (mi *K6) Group(name string, fn goja.Callable) (goja.Value, error) {
 	ret, err := fn(goja.Undefined())
 	t := time.Now()
 
-	tags := state.CloneTags()
-
 	ctx := mi.vu.Context()
+	ctm := state.Tags.GetCurrentValues()
 	metrics.PushIfNotDone(ctx, state.Samples, metrics.Sample{
-		Time:   t,
-		Metric: state.BuiltinMetrics.GroupDuration,
-		Tags:   metrics.IntoSampleTags(&tags),
-		Value:  metrics.D(t.Sub(startTime)),
+		TimeSeries: metrics.TimeSeries{
+			Metric: state.BuiltinMetrics.GroupDuration,
+			Tags:   ctm.Tags,
+		},
+		Time:     t,
+		Value:    metrics.D(t.Sub(startTime)),
+		Metadata: ctm.Metadata,
 	})
 
 	return ret, err
@@ -146,11 +152,10 @@ func (mi *K6) Check(arg0, checks goja.Value, extras ...goja.Value) (bool, error)
 	t := time.Now()
 
 	// Prepare the metric tags
-	commonTags := state.CloneTags()
+	commonTagsAndMeta := state.Tags.GetCurrentValues()
 	if len(extras) > 0 {
-		obj := extras[0].ToObject(rt)
-		for _, k := range obj.Keys() {
-			commonTags[k] = obj.Get(k).String()
+		if err := common.ApplyCustomUserTags(rt, &commonTagsAndMeta, extras[0]); err != nil {
+			return false, err
 		}
 	}
 
@@ -160,18 +165,15 @@ func (mi *K6) Check(arg0, checks goja.Value, extras ...goja.Value) (bool, error)
 	for _, name := range obj.Keys() {
 		val := obj.Get(name)
 
-		tags := make(map[string]string, len(commonTags))
-		for k, v := range commonTags {
-			tags[k] = v
-		}
-
 		// Resolve the check record.
 		check, err := state.Group.Check(name)
 		if err != nil {
 			return false, err
 		}
+
+		tags := commonTagsAndMeta.Tags
 		if state.Options.SystemTags.Has(metrics.TagCheck) {
-			tags["check"] = check.Name
+			tags = tags.With("check", check.Name)
 		}
 
 		// Resolve callables into values.
@@ -185,23 +187,29 @@ func (mi *K6) Check(arg0, checks goja.Value, extras ...goja.Value) (bool, error)
 			}
 		}
 
-		sampleTags := metrics.IntoSampleTags(&tags)
-
 		// Emit! (But only if we have a valid context.)
 		select {
 		case <-ctx.Done():
 		default:
+			sample := metrics.Sample{
+				TimeSeries: metrics.TimeSeries{
+					Metric: state.BuiltinMetrics.Checks,
+					Tags:   tags,
+				},
+				Time:     t,
+				Metadata: commonTagsAndMeta.Metadata,
+				Value:    0,
+			}
 			if val.ToBoolean() {
 				atomic.AddInt64(&check.Passes, 1)
-				metrics.PushIfNotDone(ctx, state.Samples,
-					metrics.Sample{Time: t, Metric: state.BuiltinMetrics.Checks, Tags: sampleTags, Value: 1})
+				sample.Value = 1
 			} else {
 				atomic.AddInt64(&check.Fails, 1)
-				metrics.PushIfNotDone(ctx, state.Samples,
-					metrics.Sample{Time: t, Metric: state.BuiltinMetrics.Checks, Tags: sampleTags, Value: 0})
+
 				// A single failure makes the return value false.
 				succ = false
 			}
+			metrics.PushIfNotDone(ctx, state.Samples, sample)
 		}
 
 		if exc != nil {
