@@ -133,6 +133,7 @@ func TestRandSeed(t *testing.T) {
 func TestGroup(t *testing.T) {
 	t.Parallel()
 
+	registry := metrics.NewRegistry()
 	setupGroupTest := func() (*goja.Runtime, *lib.State, *lib.Group) {
 		root, err := lib.NewGroup("", nil)
 		assert.NoError(t, err)
@@ -141,12 +142,12 @@ func TestGroup(t *testing.T) {
 		state := &lib.State{
 			Group:   root,
 			Samples: make(chan metrics.SampleContainer, 1000),
-			Tags:    lib.NewTagMap(nil),
+			Tags:    lib.NewVUStateTags(registry.RootTagSet()),
 			Options: lib.Options{
 				SystemTags: metrics.NewSystemTagSet(metrics.TagGroup),
 			},
 		}
-		state.BuiltinMetrics = metrics.RegisterBuiltinMetrics(metrics.NewRegistry())
+		state.BuiltinMetrics = metrics.RegisterBuiltinMetrics(registry)
 
 		m, ok := New().NewModuleInstance(
 			&modulestest.VU{
@@ -165,7 +166,7 @@ func TestGroup(t *testing.T) {
 		rt, state, root := setupGroupTest()
 		assert.Equal(t, state.Group, root)
 		require.NoError(t, rt.Set("fn", func() {
-			groupTag, ok := state.Tags.Get("group")
+			groupTag, ok := state.Tags.GetCurrentValues().Tags.Get("group")
 			require.True(t, ok)
 			assert.Equal(t, groupTag, "::my group")
 			assert.Equal(t, state.Group.Name, "my group")
@@ -174,7 +175,7 @@ func TestGroup(t *testing.T) {
 		_, err := rt.RunString(`k6.group("my group", fn)`)
 		assert.NoError(t, err)
 		assert.Equal(t, state.Group, root)
-		groupTag, ok := state.Tags.Get("group")
+		groupTag, ok := state.Tags.GetCurrentValues().Tags.Get("group")
 		require.True(t, ok)
 		assert.Equal(t, groupTag, root.Name)
 	})
@@ -195,6 +196,7 @@ func checkTestRuntime(t testing.TB) (*goja.Runtime, chan metrics.SampleContainer
 	require.True(t, ok)
 	require.NoError(t, rt.Set("k6", m.Exports().Named))
 
+	registry := metrics.NewRegistry()
 	root, err := lib.NewGroup("", nil)
 	assert.NoError(t, err)
 	samples := make(chan metrics.SampleContainer, 1000)
@@ -203,11 +205,9 @@ func checkTestRuntime(t testing.TB) (*goja.Runtime, chan metrics.SampleContainer
 		Options: lib.Options{
 			SystemTags: &metrics.DefaultSystemTagSet,
 		},
-		Samples: samples,
-		Tags: lib.NewTagMap(map[string]string{
-			"group": root.Path,
-		}),
-		BuiltinMetrics: metrics.RegisterBuiltinMetrics(metrics.NewRegistry()),
+		Samples:        samples,
+		Tags:           lib.NewVUStateTags(registry.RootTagSet().WithTagsFromMap(map[string]string{"group": root.Path})),
+		BuiltinMetrics: metrics.RegisterBuiltinMetrics(registry),
 	}
 	test.MoveToVUContext(state)
 
@@ -232,7 +232,7 @@ func TestCheckObject(t *testing.T) {
 		assert.Equal(t, map[string]string{
 			"group": "",
 			"check": "check",
-		}, sample.Tags.CloneTags())
+		}, sample.Tags.Map())
 	}
 
 	t.Run("Multiple", func(t *testing.T) {
@@ -291,7 +291,7 @@ func TestCheckArray(t *testing.T) {
 		assert.Equal(t, map[string]string{
 			"group": "",
 			"check": "0",
-		}, sample.Tags.CloneTags())
+		}, sample.Tags.Map())
 	}
 }
 
@@ -336,7 +336,7 @@ func TestCheckThrows(t *testing.T) {
 		assert.Equal(t, map[string]string{
 			"group": "",
 			"check": "a",
-		}, sample.Tags.CloneTags())
+		}, sample.Tags.Map())
 	}
 }
 
@@ -389,7 +389,7 @@ func TestCheckTypes(t *testing.T) {
 						assert.Equal(t, map[string]string{
 							"group": "",
 							"check": "check",
-						}, sample.Tags.CloneTags())
+						}, sample.Tags.Map())
 					}
 				})
 			}
@@ -406,18 +406,19 @@ func TestCheckContextExpiry(t *testing.T) {
 	require.NoError(t, err)
 
 	samples := make(chan metrics.SampleContainer, 1000)
+	registry := metrics.NewRegistry()
 	state := &lib.State{
 		Group: root,
 		Options: lib.Options{
 			SystemTags: &metrics.DefaultSystemTagSet,
 		},
 		Samples: samples,
-		Tags: lib.NewTagMap(map[string]string{
+		Tags: lib.NewVUStateTags(registry.RootTagSet().WithTagsFromMap(map[string]string{
 			"group": root.Path,
-		}),
+		})),
+		BuiltinMetrics: metrics.RegisterBuiltinMetrics(registry),
 	}
 
-	state.BuiltinMetrics = metrics.RegisterBuiltinMetrics(metrics.NewRegistry())
 	m, ok := New().NewModuleInstance(
 		&modulestest.VU{
 			RuntimeField: rt,
@@ -469,6 +470,6 @@ func TestCheckTags(t *testing.T) {
 			"check": "check",
 			"a":     "1",
 			"b":     "2",
-		}, sample.Tags.CloneTags())
+		}, sample.Tags.Map())
 	}
 }
