@@ -1,23 +1,3 @@
-/*
- *
- * k6 - a next-generation load testing tool
- * Copyright (C) 2019 Load Impact
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- */
-
 package executor
 
 import (
@@ -61,7 +41,7 @@ func TestExecutionStateVUIDs(t *testing.T) {
 			require.NoError(t, err)
 
 			start, offsets, _ := et.GetStripedOffsets()
-			es := lib.NewExecutionState(lib.Options{}, et, nil, 0, 0)
+			es := lib.NewExecutionState(nil, et, 0, 0)
 
 			idl, idg := es.GetUniqueVUIdentifiers()
 			assert.Equal(t, uint64(1), idl)
@@ -102,7 +82,7 @@ func TestExecutionStateGettingVUsWhenNonAreAvailable(t *testing.T) {
 	t.Parallel()
 	et, err := lib.NewExecutionTuple(nil, nil)
 	require.NoError(t, err)
-	es := lib.NewExecutionState(lib.Options{}, et, nil, 0, 0)
+	es := lib.NewExecutionState(nil, et, 0, 0)
 	logHook := &testutils.SimpleLogrusHook{HookedLevels: []logrus.Level{logrus.WarnLevel}}
 	testLog := logrus.New()
 	testLog.AddHook(logHook)
@@ -128,7 +108,7 @@ func TestExecutionStateGettingVUs(t *testing.T) {
 
 	et, err := lib.NewExecutionTuple(nil, nil)
 	require.NoError(t, err)
-	es := lib.NewExecutionState(lib.Options{}, et, nil, 10, 20)
+	es := lib.NewExecutionState(nil, et, 10, 20)
 	es.SetInitVUFunc(func(_ context.Context, _ *logrus.Entry) (lib.InitializedVU, error) {
 		return &minirunner.VU{}, nil
 	})
@@ -193,7 +173,7 @@ func TestMarkStartedPanicsOnSecondRun(t *testing.T) {
 	t.Parallel()
 	et, err := lib.NewExecutionTuple(nil, nil)
 	require.NoError(t, err)
-	es := lib.NewExecutionState(lib.Options{}, et, nil, 0, 0)
+	es := lib.NewExecutionState(nil, et, 0, 0)
 	require.False(t, es.HasStarted())
 	es.MarkStarted()
 	require.True(t, es.HasStarted())
@@ -204,7 +184,7 @@ func TestMarkEnded(t *testing.T) {
 	t.Parallel()
 	et, err := lib.NewExecutionTuple(nil, nil)
 	require.NoError(t, err)
-	es := lib.NewExecutionState(lib.Options{}, et, nil, 0, 0)
+	es := lib.NewExecutionState(nil, et, 0, 0)
 	require.False(t, es.HasEnded())
 	es.MarkEnded()
 	require.True(t, es.HasEnded())
