@@ -19,7 +19,7 @@ import (
 type transport struct {
 	ctx              context.Context
 	state            *lib.State
-	tags             map[string]string
+	tagsAndMeta      *metrics.TagsAndMeta
 	responseCallback func(int) bool
 
 	lastRequest     *unfinishedRequest
@@ -57,13 +57,13 @@ var _ http.RoundTripper = &transport{}
 func newTransport(
 	ctx context.Context,
 	state *lib.State,
-	tags map[string]string,
+	tagsAndMeta *metrics.TagsAndMeta,
 	responseCallback func(int) bool,
 ) *transport {
 	return &transport{
 		ctx:              ctx,
 		state:            state,
-		tags:             tags,
+		tagsAndMeta:      tagsAndMeta,
 		responseCallback: responseCallback,
 		lastRequestLock:  new(sync.Mutex),
 	}
@@ -75,77 +75,56 @@ func newTransport(
 func (t *transport) measureAndEmitMetrics(unfReq *unfinishedRequest) *finishedRequest {
 	trail := unfReq.tracer.Done()
 
-	tags := map[string]string{}
-	for k, v := range t.tags {
-		tags[k] = v
-	}
-
 	result := &finishedRequest{
 		unfinishedRequest: unfReq,
 		trail:             trail,
 	}
 
+	tagsAndMeta := t.tagsAndMeta.Clone()
 	enabledTags := t.state.Options.SystemTags
-	urlEnabled := enabledTags.Has(metrics.TagURL)
-	var setName bool
-	if _, ok := tags["name"]; !ok && enabledTags.Has(metrics.TagName) {
-		setName = true
-	}
-	if urlEnabled || setName {
-		cleanURL := URL{u: unfReq.request.URL, URL: unfReq.request.URL.String()}.Clean()
-		if urlEnabled {
-			tags["url"] = cleanURL
-		}
-		if setName {
-			tags["name"] = cleanURL
-		}
+	cleanURL := URL{u: unfReq.request.URL, URL: unfReq.request.URL.String()}.Clean()
+
+	// After k6 v0.41.0, the `name` and `url` tags have the exact same values:
+	nameTagValue, nameTagManuallySet := tagsAndMeta.Tags.Get(metrics.TagName.String())
+	if !nameTagManuallySet {
+		// If the user *didn't* manually set a `name` tag value and didn't use
+		// the http.url template literal helper to have k6 automatically set
+		// it (see `lib/netext/httpext.MakeRequest()`), we will use the cleaned
+		// URL value as the value of both `name` and `url` tags.
+		tagsAndMeta.SetSystemTagOrMetaIfEnabled(enabledTags, metrics.TagName, cleanURL)
+		tagsAndMeta.SetSystemTagOrMetaIfEnabled(enabledTags, metrics.TagURL, cleanURL)
+	} else {
+		// However, if the user set the `name` tag value somehow, we will use
+		// whatever they set as the value of the `url` tags too, to prevent
+		// high-cardinality values in the indexed tags.
+		tagsAndMeta.SetSystemTagOrMetaIfEnabled(enabledTags, metrics.TagURL, nameTagValue)
 	}
 
-	if enabledTags.Has(metrics.TagMethod) {
-		tags["method"] = unfReq.request.Method
-	}
+	tagsAndMeta.SetSystemTagOrMetaIfEnabled(enabledTags, metrics.TagMethod, unfReq.request.Method)
 
 	if unfReq.err != nil {
 		result.errorCode, result.errorMsg = errorCodeForError(unfReq.err)
-		if enabledTags.Has(metrics.TagError) {
-			tags["error"] = result.errorMsg
-		}
-
-		if enabledTags.Has(metrics.TagErrorCode) {
-			tags["error_code"] = strconv.Itoa(int(result.errorCode))
-		}
-
-		if enabledTags.Has(metrics.TagStatus) {
-			tags["status"] = "0"
-		}
+		tagsAndMeta.SetSystemTagOrMetaIfEnabled(enabledTags, metrics.TagError, result.errorMsg)
+		tagsAndMeta.SetSystemTagOrMetaIfEnabled(enabledTags, metrics.TagErrorCode, strconv.Itoa(int(result.errorCode)))
+		tagsAndMeta.SetSystemTagOrMetaIfEnabled(enabledTags, metrics.TagStatus, "0")
 	} else {
-		if enabledTags.Has(metrics.TagStatus) {
-			tags["status"] = strconv.Itoa(unfReq.response.StatusCode)
-		}
+		tagsAndMeta.SetSystemTagOrMetaIfEnabled(enabledTags, metrics.TagStatus, strconv.Itoa(unfReq.response.StatusCode))
 		if unfReq.response.StatusCode >= 400 {
-			if enabledTags.Has(metrics.TagErrorCode) {
-				result.errorCode = errCode(1000 + unfReq.response.StatusCode)
-				tags["error_code"] = strconv.Itoa(int(result.errorCode))
-			}
-		}
-		if enabledTags.Has(metrics.TagProto) {
-			tags["proto"] = unfReq.response.Proto
+			result.errorCode = errCode(1000 + unfReq.response.StatusCode)
+			tagsAndMeta.SetSystemTagOrMetaIfEnabled(enabledTags, metrics.TagErrorCode, strconv.Itoa(int(result.errorCode)))
 		}
+		tagsAndMeta.SetSystemTagOrMetaIfEnabled(enabledTags, metrics.TagProto, unfReq.response.Proto)
 
 		if unfReq.response.TLS != nil {
 			tlsInfo, oscp := netext.ParseTLSConnState(unfReq.response.TLS)
-			if enabledTags.Has(metrics.TagTLSVersion) {
-				tags["tls_version"] = tlsInfo.Version
-			}
-			if enabledTags.Has(metrics.TagOCSPStatus) {
-				tags["ocsp_status"] = oscp.Status
-			}
+			tagsAndMeta.SetSystemTagOrMetaIfEnabled(enabledTags, metrics.TagTLSVersion, tlsInfo.Version)
+			tagsAndMeta.SetSystemTagOrMetaIfEnabled(enabledTags, metrics.TagOCSPStatus, oscp.Status)
 			result.tlsInfo = tlsInfo
 		}
 	}
 	if enabledTags.Has(metrics.TagIP) && trail.ConnRemoteAddr != nil {
 		if ip, _, err := net.SplitHostPort(trail.ConnRemoteAddr.String()); err == nil {
-			tags["ip"] = ip
+			tagsAndMeta.SetSystemTagOrMeta(metrics.TagIP, ip)
 		}
 	}
 	var failed float64
@@ -159,14 +138,10 @@ func (t *transport) measureAndEmitMetrics(unfReq *unfinishedRequest) *finishedRe
 			failed = 1
 		}
 
-		if enabledTags.Has(metrics.TagExpectedResponse) {
-			tags[metrics.TagExpectedResponse.String()] = strconv.FormatBool(expected)
-		}
+		tagsAndMeta.SetSystemTagOrMetaIfEnabled(enabledTags, metrics.TagExpectedResponse, strconv.FormatBool(expected))
 	}
 
-	finalTags := metrics.IntoSampleTags(&tags)
-	builtinMetrics := t.state.BuiltinMetrics
-	trail.SaveSamples(builtinMetrics, finalTags)
+	trail.SaveSamples(t.state.BuiltinMetrics, &tagsAndMeta)
 	if t.responseCallback != nil {
 		trail.Failed.Valid = true
 		if failed == 1 {
@@ -174,12 +149,17 @@ func (t *transport) measureAndEmitMetrics(unfReq *unfinishedRequest) *finishedRe
 		}
 		trail.Samples = append(trail.Samples,
 			metrics.Sample{
-				Metric: builtinMetrics.HTTPReqFailed, Time: trail.EndTime, Tags: finalTags, Value: failed,
+				TimeSeries: metrics.TimeSeries{
+					Metric: t.state.BuiltinMetrics.HTTPReqFailed,
+					Tags:   tagsAndMeta.Tags,
+				},
+				Time:     trail.EndTime,
+				Metadata: tagsAndMeta.Metadata,
+				Value:    failed,
 			},
 		)
 	}
 	metrics.PushIfNotDone(t.ctx, t.state.Samples, trail)
-
 	return result
 }
 
