@@ -4,6 +4,7 @@ import (
 	"encoding/json"
 	"fmt"
 	"math/rand"
+	"strconv"
 	"testing"
 	"time"
 
@@ -18,7 +19,8 @@ import (
 func TestSampleMarshaling(t *testing.T) {
 	t.Parallel()
 
-	builtinMetrics := metrics.RegisterBuiltinMetrics(metrics.NewRegistry())
+	registry := metrics.NewRegistry()
+	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	now := time.Now()
 	exptoMicroSecond := now.UnixNano() / 1000
 
@@ -33,7 +35,7 @@ func TestSampleMarshaling(t *testing.T) {
 				Data: &SampleDataSingle{
 					Type:  builtinMetrics.VUs.Type,
 					Time:  toMicroSecond(now),
-					Tags:  metrics.IntoSampleTags(&map[string]string{"aaa": "bbb", "ccc": "123"}),
+					Tags:  []byte(`{"aaa":"bbb","ccc":"123"}`),
 					Value: 999,
 				},
 			},
@@ -45,7 +47,7 @@ func TestSampleMarshaling(t *testing.T) {
 				Metric: "iter_li_all",
 				Data: &SampleDataMap{
 					Time: toMicroSecond(now),
-					Tags: metrics.IntoSampleTags(&map[string]string{"test": "mest"}),
+					Tags: []byte(`{"test":"mest"}`),
 					Values: map[string]float64{
 						metrics.DataSentName:          1234.5,
 						metrics.DataReceivedName:      6789.1,
@@ -65,6 +67,7 @@ func TestSampleMarshaling(t *testing.T) {
 				Sending:        4000,
 				Waiting:        5000,
 				Receiving:      6000,
+				Tags:           registry.RootTagSet(),
 			}),
 			fmt.Sprintf(`{"type":"Points","metric":"http_req_li_all","data":{"time":"%d","type":"counter","values":{"http_req_blocked":0.001,"http_req_connecting":0.002,"http_req_duration":0.123,"http_req_receiving":0.006,"http_req_sending":0.004,"http_req_tls_handshaking":0.003,"http_req_waiting":0.005,"http_reqs":1}}}`, exptoMicroSecond),
 		},
@@ -79,6 +82,7 @@ func TestSampleMarshaling(t *testing.T) {
 				Waiting:        5000,
 				Receiving:      6000,
 				Failed:         null.NewBool(false, true),
+				Tags:           registry.RootTagSet(),
 			}),
 			fmt.Sprintf(`{"type":"Points","metric":"http_req_li_all","data":{"time":"%d","type":"counter","values":{"http_req_blocked":0.001,"http_req_connecting":0.002,"http_req_duration":0.123,"http_req_failed":0,"http_req_receiving":0.006,"http_req_sending":0.004,"http_req_tls_handshaking":0.003,"http_req_waiting":0.005,"http_reqs":1}}}`, exptoMicroSecond),
 		},
@@ -87,7 +91,7 @@ func TestSampleMarshaling(t *testing.T) {
 				aggrData := &SampleDataAggregatedHTTPReqs{
 					Time: exptoMicroSecond,
 					Type: "aggregated_trend",
-					Tags: metrics.IntoSampleTags(&map[string]string{"test": "mest"}),
+					Tags: []byte(`{"test":"mest"}`),
 				}
 				aggrData.Add(
 					&httpext.Trail{
@@ -129,7 +133,7 @@ func TestSampleMarshaling(t *testing.T) {
 				aggrData := &SampleDataAggregatedHTTPReqs{
 					Time: exptoMicroSecond,
 					Type: "aggregated_trend",
-					Tags: metrics.IntoSampleTags(&map[string]string{"test": "mest"}),
+					Tags: []byte(`{"test": "mest"}`),
 				}
 				aggrData.Add(
 					&httpext.Trail{
@@ -169,13 +173,13 @@ func TestSampleMarshaling(t *testing.T) {
 		},
 	}
 
-	for _, tc := range testCases {
+	for i, tc := range testCases {
 		sJSON, err := easyjson.Marshal(tc.s)
 		if !assert.NoError(t, err) {
 			continue
 		}
 		t.Logf(string(sJSON))
-		assert.JSONEq(t, tc.json, string(sJSON))
+		assert.JSONEq(t, tc.json, string(sJSON), "testcase"+strconv.Itoa(i))
 
 		var newS Sample
 		assert.NoError(t, json.Unmarshal(sJSON, &newS))
