@@ -35,17 +35,19 @@ func TestMakeHeader(t *testing.T) {
 		testname, tags := testname, tags
 		t.Run(testname, func(t *testing.T) {
 			header := MakeHeader(tags)
-			assert.Equal(t, len(tags)+4, len(header))
+			assert.Equal(t, len(tags)+5, len(header))
 			assert.Equal(t, "metric_name", header[0])
 			assert.Equal(t, "timestamp", header[1])
 			assert.Equal(t, "metric_value", header[2])
-			assert.Equal(t, "extra_tags", header[len(header)-1])
+			assert.Equal(t, "extra_tags", header[len(header)-2])
+			assert.Equal(t, "metadata", header[len(header)-1])
 		})
 	}
 }
 
 func TestSampleToRow(t *testing.T) {
-	testMetric, err := metrics.NewRegistry().NewMetric("my_metric", metrics.Gauge)
+	registry := metrics.NewRegistry()
+	testMetric, err := registry.NewMetric("my_metric", metrics.Gauge)
 	require.NoError(t, err)
 
 	testData := []struct {
@@ -58,14 +60,16 @@ func TestSampleToRow(t *testing.T) {
 		{
 			testname: "One res tag, one ignored tag, one extra tag",
 			sample: &metrics.Sample{
-				Time:   time.Unix(1562324644, 0),
-				Metric: testMetric,
-				Value:  1,
-				Tags: metrics.NewSampleTags(map[string]string{
-					"tag1": "val1",
-					"tag2": "val2",
-					"tag3": "val3",
-				}),
+				TimeSeries: metrics.TimeSeries{
+					Metric: testMetric,
+					Tags: registry.RootTagSet().WithTagsFromMap(map[string]string{
+						"tag1": "val1",
+						"tag2": "val2",
+						"tag3": "val3",
+					}),
+				},
+				Time:  time.Unix(1562324644, 0),
+				Value: 1,
 			},
 			resTags:     []string{"tag1"},
 			ignoredTags: []string{"tag2"},
@@ -74,16 +78,18 @@ func TestSampleToRow(t *testing.T) {
 		{
 			testname: "Two res tags, three extra tags",
 			sample: &metrics.Sample{
-				Time:   time.Unix(1562324644, 0),
-				Metric: testMetric,
-				Value:  1,
-				Tags: metrics.NewSampleTags(map[string]string{
-					"tag1": "val1",
-					"tag2": "val2",
-					"tag3": "val3",
-					"tag4": "val4",
-					"tag5": "val5",
-				}),
+				TimeSeries: metrics.TimeSeries{
+					Metric: testMetric,
+					Tags: registry.RootTagSet().WithTagsFromMap(map[string]string{
+						"tag1": "val1",
+						"tag2": "val2",
+						"tag3": "val3",
+						"tag4": "val4",
+						"tag5": "val5",
+					}),
+				},
+				Time:  time.Unix(1562324644, 0),
+				Value: 1,
 			},
 			resTags:     []string{"tag1", "tag2"},
 			ignoredTags: []string{},
@@ -92,17 +98,19 @@ func TestSampleToRow(t *testing.T) {
 		{
 			testname: "Two res tags, two ignored, with RFC3339 timestamp",
 			sample: &metrics.Sample{
-				Time:   time.Unix(1562324644, 0),
-				Metric: testMetric,
-				Value:  1,
-				Tags: metrics.NewSampleTags(map[string]string{
-					"tag1": "val1",
-					"tag2": "val2",
-					"tag3": "val3",
-					"tag4": "val4",
-					"tag5": "val5",
-					"tag6": "val6",
-				}),
+				TimeSeries: metrics.TimeSeries{
+					Metric: testMetric,
+					Tags: registry.RootTagSet().WithTagsFromMap(map[string]string{
+						"tag1": "val1",
+						"tag2": "val2",
+						"tag3": "val3",
+						"tag4": "val4",
+						"tag5": "val5",
+						"tag6": "val6",
+					}),
+				},
+				Time:  time.Unix(1562324644, 0),
+				Value: 1,
 			},
 			resTags:     []string{"tag1", "tag3"},
 			ignoredTags: []string{"tag4", "tag6"},
@@ -162,12 +170,12 @@ func TestSampleToRow(t *testing.T) {
 		expectedRow := expected[i]
 
 		t.Run(testname, func(t *testing.T) {
-			row := SampleToRow(sample, resTags, ignoredTags, make([]string, 3+len(resTags)+1), timeFormat)
+			row := SampleToRow(sample, resTags, ignoredTags, make([]string, 3+len(resTags)+2), timeFormat)
 			for ind, cell := range expectedRow.baseRow {
 				assert.Equal(t, cell, row[ind])
 			}
 			for _, cell := range expectedRow.extraRow {
-				assert.Contains(t, row[len(row)-1], cell)
+				assert.Contains(t, row[len(row)-2], cell)
 			}
 		})
 	}
@@ -204,7 +212,8 @@ func readCompressedFile(fileName string, fs afero.Fs) string {
 func TestRun(t *testing.T) {
 	t.Parallel()
 
-	testMetric, err := metrics.NewRegistry().NewMetric("my_metric", metrics.Gauge)
+	registry := metrics.NewRegistry()
+	testMetric, err := registry.NewMetric("my_metric", metrics.Gauge)
 	require.NoError(t, err)
 
 	testData := []struct {
@@ -217,91 +226,110 @@ func TestRun(t *testing.T) {
 		{
 			samples: []metrics.SampleContainer{
 				metrics.Sample{
-					Time:   time.Unix(1562324643, 0),
-					Metric: testMetric,
-					Value:  1,
-					Tags: metrics.NewSampleTags(map[string]string{
-						"check": "val1",
-						"url":   "val2",
-						"error": "val3",
-					}),
+					TimeSeries: metrics.TimeSeries{
+						Metric: testMetric,
+						Tags: registry.RootTagSet().WithTagsFromMap(map[string]string{
+							"check": "val1",
+							"url":   "val2",
+							"error": "val3",
+						}),
+					},
+					Time:  time.Unix(1562324643, 0),
+					Value: 1,
 				},
 				metrics.Sample{
-					Time:   time.Unix(1562324644, 0),
-					Metric: testMetric,
-					Value:  1,
-					Tags: metrics.NewSampleTags(map[string]string{
-						"check": "val1",
-						"url":   "val2",
-						"error": "val3",
-						"tag4":  "val4",
-					}),
+					TimeSeries: metrics.TimeSeries{
+						Metric: testMetric,
+						Tags: registry.RootTagSet().WithTagsFromMap(map[string]string{
+							"check": "val1",
+							"url":   "val2",
+							"error": "val3",
+							"tag4":  "val4",
+						}),
+					},
+					Metadata: map[string]string{},
+					Time:     time.Unix(1562324644, 0),
+					Value:    1,
 				},
 			},
 			fileName:       "test",
 			fileReaderFunc: readUnCompressedFile,
 			timeFormat:     "",
-			outputContent:  "metric_name,timestamp,metric_value,check,error,extra_tags\n" + "my_metric,1562324643,1.000000,val1,val3,url=val2\n" + "my_metric,1562324644,1.000000,val1,val3,tag4=val4&url=val2\n",
+			outputContent: "metric_name,timestamp,metric_value,check,error,extra_tags,metadata\n" +
+				"my_metric,1562324643,1.000000,val1,val3,url=val2,\n" +
+				"my_metric,1562324644,1.000000,val1,val3,tag4=val4&url=val2,\n",
 		},
 		{
 			samples: []metrics.SampleContainer{
 				metrics.Sample{
-					Time:   time.Unix(1562324643, 0),
-					Metric: testMetric,
-					Value:  1,
-					Tags: metrics.NewSampleTags(map[string]string{
-						"check": "val1",
-						"url":   "val2",
-						"error": "val3",
-					}),
+					TimeSeries: metrics.TimeSeries{
+						Metric: testMetric,
+						Tags: registry.RootTagSet().WithTagsFromMap(map[string]string{
+							"check": "val1",
+							"url":   "val2",
+							"error": "val3",
+						}),
+					},
+					Time:     time.Unix(1562324643, 0),
+					Metadata: map[string]string{"y": "1"},
+					Value:    1,
 				},
 				metrics.Sample{
-					Time:   time.Unix(1562324644, 0),
-					Metric: testMetric,
-					Value:  1,
-					Tags: metrics.NewSampleTags(map[string]string{
-						"check": "val1",
-						"url":   "val2",
-						"error": "val3",
-						"name":  "val4",
-					}),
+					TimeSeries: metrics.TimeSeries{
+						Metric: testMetric,
+						Tags: registry.RootTagSet().WithTagsFromMap(map[string]string{
+							"check": "val1",
+							"url":   "val2",
+							"error": "val3",
+							"name":  "val4",
+						}),
+					},
+					Time:  time.Unix(1562324644, 0),
+					Value: 1,
 				},
 			},
 			fileName:       "test.gz",
 			fileReaderFunc: readCompressedFile,
 			timeFormat:     "unix",
-			outputContent:  "metric_name,timestamp,metric_value,check,error,extra_tags\n" + "my_metric,1562324643,1.000000,val1,val3,url=val2\n" + "my_metric,1562324644,1.000000,val1,val3,name=val4&url=val2\n",
+			outputContent: "metric_name,timestamp,metric_value,check,error,extra_tags,metadata\n" +
+				"my_metric,1562324643,1.000000,val1,val3,url=val2,y=1\n" +
+				"my_metric,1562324644,1.000000,val1,val3,name=val4&url=val2,\n",
 		},
 		{
 			samples: []metrics.SampleContainer{
 				metrics.Sample{
-					Time:   time.Unix(1562324644, 0),
-					Metric: testMetric,
-					Value:  1,
-					Tags: metrics.NewSampleTags(map[string]string{
-						"check": "val1",
-						"url":   "val2",
-						"error": "val3",
-					}),
+					TimeSeries: metrics.TimeSeries{
+						Metric: testMetric,
+						Tags: registry.RootTagSet().WithTagsFromMap(map[string]string{
+							"check": "val1",
+							"url":   "val2",
+							"error": "val3",
+						}),
+					},
+					Time:  time.Unix(1562324644, 0),
+					Value: 1,
 				},
 				metrics.Sample{
-					Time:   time.Unix(1562324644, 0),
-					Metric: testMetric,
-					Value:  1,
-					Tags: metrics.NewSampleTags(map[string]string{
-						"check": "val1",
-						"url":   "val2",
-						"error": "val3",
-						"name":  "val4",
-					}),
+					TimeSeries: metrics.TimeSeries{
+						Metric: testMetric,
+						Tags: registry.RootTagSet().WithTagsFromMap(map[string]string{
+							"check": "val1",
+							"url":   "val2",
+							"error": "val3",
+							"name":  "val4",
+						}),
+					},
+					Time:     time.Unix(1562324644, 0),
+					Metadata: map[string]string{"y": "2", "z": "3"},
+					Value:    1,
 				},
 			},
 			fileName:       "test",
 			fileReaderFunc: readUnCompressedFile,
 			timeFormat:     "rfc3339",
-			outputContent: "metric_name,timestamp,metric_value,check,error,extra_tags\n" +
-				"my_metric," + time.Unix(1562324644, 0).Format(time.RFC3339) + ",1.000000,val1,val3,url=val2\n" +
-				"my_metric," + time.Unix(1562324644, 0).Format(time.RFC3339) + ",1.000000,val1,val3,name=val4&url=val2\n",
+			outputContent: "metric_name,timestamp,metric_value,check,error,extra_tags,metadata\n" +
+				"my_metric," + time.Unix(1562324644, 0).Format(time.RFC3339) + ",1.000000,val1,val3,url=val2,\n" +
+				"my_metric," + time.Unix(1562324644, 0).Format(time.RFC3339) + ",1.000000,val1,val3,name=val4&url=val2,y=2&z=3\n",
 		},
 	}
 
@@ -330,7 +358,6 @@ func TestRun(t *testing.T) {
 
 			require.NoError(t, output.Start())
 			output.AddMetricSamples(data.samples)
-			time.Sleep(1 * time.Second)
 			require.NoError(t, output.Stop())
 
 			finalOutput := data.fileReaderFunc(data.fileName, mem)
@@ -345,9 +372,12 @@ func sortExtraTagsForTest(t *testing.T, input string) string {
 	lines, err := r.ReadAll()
 	require.NoError(t, err)
 	for i, line := range lines[1:] {
-		extraTags := strings.Split(line[len(line)-1], "&")
+		extraTags := strings.Split(line[len(line)-2], "&")
 		sort.Strings(extraTags)
-		lines[i+1][len(line)-1] = strings.Join(extraTags, "&")
+		lines[i+1][len(line)-2] = strings.Join(extraTags, "&")
+		extraMetadata := strings.Split(line[len(line)-1], "&")
+		sort.Strings(extraMetadata)
+		lines[i+1][len(line)-1] = strings.Join(extraMetadata, "&")
 	}
 	var b bytes.Buffer
 	w := csv.NewWriter(&b)
